@@ -3,11 +3,15 @@ from copy import deepcopy
 
 import networkx as nx
 import pytest
-from lxml import etree as ET
 
 import geff.interops.trackmate_xml as tm_xml
 from geff.utils import nx_is_equal, validate
 
+try:
+    from lxml import etree as ET
+except ImportError:
+    import xml.etree.ElementTree as ET
+
 
 def test_get_units():
     space_warning = "No space unit found in the XML file. Setting to 'pixel'."
